;	Elite VIC 20 Graphics Routines by Aleksi Eeben (aleksi.eeben@gmail.com) 19 Dec 2024

	processor 6502


;	Zero page

ptr	equ	$00		; bitmap pointer
ptrh	equ	$01

src	equ	$02		; source address in InitFont, Print
srch	equ	$03
x2	equ	$02		; also x2, y2 in Line
y2	equ	$03
stripes	equ	$02		; also stripe count and repeating value in ShowPanel
repval	equ	$03


	org	$0400,$00

;	Jump vectors

	jmp	Init		; $0400 init VIC-I, bitmap and charset
	jmp	Plot		; $0403 eor plot at x,y
	jmp	Line		; $0406 eor line from x,y to x2,y2 (zeropage)
	jmp	Cursor		; $0409 set print cursor at x,y (0-31, 0-24)
	jmp	Print		; $040c print null-terminated string from YYAA at cursor
	jmp	DrawBox		; $040f draw screen box
	jmp	ShowPanel	; $0412 show ship control panel
	jmp	HorizLine	; $0415 draw horizontal line at y
	jmp	ClearBox	; $0418 clear screen box (also clears bitmap)
	jmp	ClearBitmap	; $041b clear bitmap only


	org	$0428,$00

;	Tests

	jmp	TestLines	; $0428	(sys1064)
	jmp	TestData	; $042b (sys1067)
	jmp	TestPlots	; $042e (sys1070)


;	Set up VIC-I, charset and bitmap

Init	subroutine
	ldy	#$00
	sty	$9002
	lda	#$01
.2
	sta	$9400,y		; init color RAM
	iny
	bne	.2

	jsr	ClearBitmap
	jsr	InitFont

	ldy	#$17		; create characters for left/right side
.3
	lda	#$01
	sta	$1e00,y
	lda	#$80
	sta	$1e20,y
	lda	#$00		; space character used for hiding left/right side
	sta	$10f0,y		; (clears a few extra bytes)
	dey
	bpl	.3

	ldy	#$07		; only 8 pix used on 13th row (12*16+8 = 200)
	lda	#$00
.4
	sta	$1e18,y
	sta	$1e38,y
	dey
	bpl	.4

	ldy	#$01		; bitmap layout
	lda	#$10
.nextrow
	ldx	#$10		; 16 characters per line
.5
	sta	$1000,y
	clc
	adc	#$0d
	iny
	dex
	bne	.5

	sec			; character at top of next stripe
	sbc	#$cf

	iny			; skip side line characters
	iny

	cpy	#$eb		; bottom of screen 13*18
	bne	.nextrow

	lda	#$cc		; screen matrix at $1000-$10e9 (18x13) and
	sta	$9005		; chars at $1000 ($1100-$1dff used for bitmap, $1e00-$1e3f for side lines)

	lda	#$08		; screen black
	sta	$900f

	ldy	#$03		; set screen position and size
.1
	lda	$ede4,y		; values relative to PAL/NTSC-specific setup in ROM
	clc
	adc	.scroffset,y
	sta	$9000,y
	lda	#$00		; stop sounds
	sta	$900a,y
	dey
	bpl	.1

	rts

.scroffset
	dc.b	$04,$fc,$fc,$ed		; centered screen, 18x13 characters, 8x16 pixel size


;	EOR pixel at x,y (0-127, 0-199)

Plot	subroutine
;	cpy	#200		; optional y bounds check
;	bcs	.out

	lda	xcol,x
	sta	ptr
	lda	xcolh,x
	sta	ptrh
	lda	(ptr),y
	eor	xbit,x
	sta	(ptr),y

.out
	rts


;	Draw EOR line from x,y to x2,y2 (x,y in registers, set x2,y2 on zeropage)
;	Adapted from Hein's linedraw routine on Codebase64

Line	subroutine
	stx	x1
	sty	y1

	ldx	#$e8	; inx
	lda	x2
	sta	tox+1
	sec
	sbc	x1
	bcs	.1
	eor	#$ff
	adc	#$01
	ldx	#$ca	; dex
.1
	sta	dy+1
	sta	ty1+1
	sta	ty2+1
	stx	incx1
	stx	incx2

	ldx	#$c8	; iny
	lda	y2
	sta	toy+1
	sec
	sbc	y1
	bcs	.2
	eor	#$ff
	adc	#$01
	ldx	#$88	; dey
.2
	stx	incy1
	stx	incy2

	ldx	x1

dy
	cmp	#0
	bcc	steep

	sta	tx1+1
	lsr
	sta	errx+1
loopx
	lda	xcol,x
	sta	ptr
	lda	xcolh,x
	sta	ptrh
	lda	(ptr),y
	eor	xbit,x
	sta	(ptr),y

errx	lda	#$00
	sec
ty1	sbc	#$00
	bcs	.3

tx1	adc	#$00
incx1	inx
.3
	sta	errx+1

incy1	iny
toy	cpy	#$00
	bne	loopx
	rts

steep
	sta	tx2+1
	lsr
	sta	erry+1
loopy
	lda	xcol,x
	sta	ptr
	lda	xcolh,x
	sta	ptrh
	lda	(ptr),y
	eor	xbit,x
	sta	(ptr),y

erry	lda	#$00
	sec
tx2	sbc	#$00
	bcs	.4

ty2	adc	#$00
incy2	iny
.4
	sta	erry+1

incx2	inx
tox	cpx	#$00
	bne	loopy

	rts


x1	dc.b	$00
y1	dc.b	$00


;	Set location of print cursor to x,y (0-31, 0-24)

Cursor	subroutine
	cpy	#25		; check screen bounds
	bcs	.out
	cpx	#32
	bcc	.ok
.out
	rts
.ok
	pha

	stx	.cursorx	; store x-coordinate for cr/new line
	txa
	and	#$fe
	asl
	asl
	tax

	sty	.cursory	; store y-coordinate for cr/new line
	tya
	asl
	asl
	asl
	adc	xcol,x
	sta	ptr
	lda	xcolh,x
	adc	#$00
	sta	ptrh

	ldx	.cursorx
	pla

	rts

.cursorx	dc.b	$00
.cursory	dc.b	$00


;	Print null-terminated ASCII string from YYAA to cursor location

Print
	sty	.nextchar+2	; string location in memory
	sta	.nextchar+1

	pha
	tya
	pha
	txa
	pha

	ldx	.cursorx

	ldy	#$00
.nextchar
	lda	$1000,y		; get next character of string
	beq	.end
	cmp	#13
	beq	.return

	asl			; calculate pointer to character data
	sta	src
	lda	#$25		; ASCII $20-$7f in low nybbles at $9500-$97ff ($25<<2 = $94)
	asl	src
	rol
	asl	src
	rol
	sta	srch

	tya
	pha

	txa
	lsr
	bcs	.oddcol

.evencol
	ldy	#$07
.cp1
	lda	(src),y		; eor character to even column
	asl
	asl
	asl
	asl
	eor	(ptr),y
	sta	(ptr),y
	dey
	bpl	.cp1

.charok
	pla
	tay
	inx
	cpx	#32
	beq	.end
	iny
	bne	.nextchar
	inc	.nextchar+2
	bne	.nextchar
.end
	pla
	tax
	pla
	tay
	pla
	rts

.oddcol
	ldy	#$07
.cp2
	lda	(src),y		; eor character to odd column
	and	#$0f
	eor	(ptr),y
	sta	(ptr),y
	dey
	bpl	.cp2

	lda	ptr		; to next char in bitmap
	clc
	adc	#$d0
	sta	ptr
	bcc	.nc
	inc	ptrh
.nc
	jmp	.charok

.return
	tya
	pha
	ldx	.cursorx
	ldy	.cursory
	iny
	cpy	#25
	bne	.nowr
	ldy	#0
.nowr
	jsr	Cursor
	pla
	tay
	iny
	jmp	.nextchar


;	Clear bitmap $1100-$1dff (does not clear the side lines)

ClearBitmap	subroutine
	lda	#$11
	sta	ptrh
	lda	#$00
	sta	ptr
	tay
	ldx	#$0d
.1
	sta	(ptr),y
	iny
	sta	(ptr),y
	iny
;	sta	(ptr),y
;	iny
;	sta	(ptr),y
;	iny
	bne	.1
	inc	ptrh
	dex
	bne	.1

	rts


;	Draw screen box

DrawBox	subroutine
	ldy	#0		; top line
	jsr	HorizLine
	ldy	#199		; bottom line
	jsr	HorizLine

	ldy	#$00
.1
	lda	#$e0		; left side
	sta	$1000,y
	lda	#$e2		; right side
	sta	$1011,y

	tya
	clc
	adc	#$12
	tay
	cpy	#$d8
	bne	.1

	lda	#$e1		; last 8 pix of left side
	sta	$10d8
	lda	#$e3		; last 8 pix of right side
	sta	$10e9

	rts


;	Hide screen box (clear bitmap and hide sides)

ClearBox	subroutine
	jsr	ClearBitmap

	ldy	#$00
.1
	lda	#$0f		; hide side lines
	sta	$1000,y
	sta	$1011,y

	tya
	clc
	adc	#$12
	tay
	cpy	#$ea
	bne	.1

	rts


;	Draw horizontal line at y (0-199)

HorizLine	subroutine
	cpy	#200
	bcs	.out
	sty	y2
	ldx	#128
	stx	x2
	ldx	#0
	jsr	Line
.out
	rts


;	Show ship control panel

ShowPanel	subroutine
	lda	#<Panel
	sta	.next+1
	lda	#>Panel
	sta	.next+2
	ldx	#$00

	lda	#$90
	sta	ptr
	lda	#$11
	sta	ptrh

	lda	#$10
	sta	stripes

.nextstripe
	ldy	#$00

.lp
	jsr	getbyte
	sta	(ptr),y
	iny
	cpy	#$38
	bne	.lp

	lda	ptr
	clc
	adc	#$d0
	sta	ptr
	bcc	.1
	inc	ptrh
.1
	dec	stripes
	bne	.nextstripe

	rts

getbyte
	dex			; repeating values
	bmi	.next
.reps
	lda	repval
	rts

.next
	lda	$1000		; get next byte or rep code
	inc	.next+1
	bne	.2
	inc	.next+2
.2
	pha
	cmp	#$d0
	bcs	.mayberle
.single
	ldx	#$00		; single value
	pla
	rts

.mayberle
	lsr
	lsr
	lsr
	lsr
	tax
	lda	repvals-$0d,x
	sta	repval
	pla
	pha
	and	#$0f
	cmp	#$0e		; $de-$df, $ee-$ef, $fe-$ff are literals
	bcs	.single

	tax			; repeats
	pla
	inx
	bne	.reps		; always branch

repvals
	dc.b	$00,$01,$80	; bytes with runs of 2-15 in panel bitmap encoded as $d0-$dd, $e0-$ed, $f0-$fd


;	Init font from packed nybbles to unused color RAM (could also load charset-bytes.bin directly to $9500)

InitFont	subroutine
	lda	#<Charset
	sta	src
	lda	#>Charset
	sta	srch

	lda	#$95
	sta	ptrh
	ldy	#$00
	sty	ptr

.1
	lda	(src),y
	jsr	.twobytes
	iny
	bne	.1

	inc	srch
.2
	lda	(src),y
	jsr	.twobytes
	iny
	bpl	.2

	rts

.twobytes
	pha
	lsr
	lsr
	lsr
	lsr
	jsr	.byteout
	pla
	and	#$0f
	;
.byteout
	sta	(ptr,x)
	asl
	asl
	asl
	asl
	inc	ptr
	bne	.noc
	inc	ptrh
.noc		
	rts


;	Tests

	include	"tests.s"


;	Data

Panel
	incbin	"panel.bin"

Charset
	incbin	"charset.bin"


	org	$0e80,$00

xcol
	ds.b	$08,<($1100+$d0*$00)
	ds.b	$08,<($1100+$d0*$01)
	ds.b	$08,<($1100+$d0*$02)
	ds.b	$08,<($1100+$d0*$03)
	ds.b	$08,<($1100+$d0*$04)
	ds.b	$08,<($1100+$d0*$05)
	ds.b	$08,<($1100+$d0*$06)
	ds.b	$08,<($1100+$d0*$07)
	ds.b	$08,<($1100+$d0*$08)
	ds.b	$08,<($1100+$d0*$09)
	ds.b	$08,<($1100+$d0*$0a)
	ds.b	$08,<($1100+$d0*$0b)
	ds.b	$08,<($1100+$d0*$0c)
	ds.b	$08,<($1100+$d0*$0d)
	ds.b	$08,<($1100+$d0*$0e)
	ds.b	$08,<($1100+$d0*$0f)

xcolh
	ds.b	$08,>($1100+$d0*$00)
	ds.b	$08,>($1100+$d0*$01)
	ds.b	$08,>($1100+$d0*$02)
	ds.b	$08,>($1100+$d0*$03)
	ds.b	$08,>($1100+$d0*$04)
	ds.b	$08,>($1100+$d0*$05)
	ds.b	$08,>($1100+$d0*$06)
	ds.b	$08,>($1100+$d0*$07)
	ds.b	$08,>($1100+$d0*$08)
	ds.b	$08,>($1100+$d0*$09)
	ds.b	$08,>($1100+$d0*$0a)
	ds.b	$08,>($1100+$d0*$0b)
	ds.b	$08,>($1100+$d0*$0c)
	ds.b	$08,>($1100+$d0*$0d)
	ds.b	$08,>($1100+$d0*$0e)
	ds.b	$08,>($1100+$d0*$0f)

xbit
	REPEAT 16
	dc.b	$80,$40,$20,$10,$08,$04,$02,$01
	REPEND

;	ds.b	$80,$00		; plot nothing if x => 128 (can be removed if x-coordinates trusted)
